from parser import Parser
from objects import Terminal, Nonterminal, State, DFA, REG
from objects import Operation as op


def test_dfa():
    q_0 = State("q_0")
    q_1 = State("q_1")
    a = Terminal("a")
    states = {q_0, q_1}
    init = q_0
    final = {q_1}
    terminals = {a}
    transition = dict()
    transition[q_0, a] = q_1
    transition[q_1, a] = q_1
    dfa = DFA(states, terminals, transition, init, final)

    return dfa

def test_dfa2():
    r_0 = State("r_0")
    r_1 = State("r_1")
    r_2 = State("r_2")
    a = Terminal("a")
    b = Terminal("b")
    states = {r_0, r_1, r_2}
    init = r_0
    final = {r_2}
    terminals = {a, b}
    transition = dict()
    transition[r_0, a] = r_1
    transition[r_0, b] = r_1
    transition[r_1, a] = r_2
    transition[r_1, b] = r_2
    dfa = DFA(states, terminals, transition, init, final)

    return dfa


def test_dfa_min():
    states = set()
    st = {}
    for i in range(1,8):
        state = State(str(i))
        st[str(i)] = state
        states.add(state)

    a = Terminal("a")
    b = Terminal("b")

    init = st[str(1)]
    final = {st[str(3)], st[str(5)], st[str(6)]}
    terminals = {a, b}
    transition = dict()
    transition[st[str(1)], a] = st[str(2)]
    transition[st[str(2)], a] = st[str(3)]
    transition[st[str(2)], b] = st[str(4)]
    transition[st[str(3)], a] = st[str(6)]
    transition[st[str(3)], b] = st[str(5)]
    transition[st[str(4)], a] = st[str(3)]
    transition[st[str(4)], b] = st[str(2)]
    transition[st[str(5)], a] = st[str(6)]
    transition[st[str(5)], b] = st[str(3)]
    transition[st[str(6)], a] = st[str(2)]
    transition[st[str(7)], a] = st[str(6)]
    transition[st[str(7)], b] = st[str(1)]
    dfa = DFA(states, terminals, transition, init, final)

    return dfa

def test_reg():
    S = Nonterminal("S")
    A = Nonterminal("A")
    a = Terminal("a")
    nonterminals = {S, A}
    init = S
    terminals = {a}
    rules = dict()
    rules[S] = {(a, S), (a)}
    rules[A] = {(a)}
    reg = REG(nonterminals, terminals, rules, init)

    return reg

def test_reg2():
    S = Nonterminal("S")
    A = Nonterminal("A")
    a = Terminal("a")
    nonterminals = {S, A}
    init = S
    terminals = {a}
    rules = dict()
    reg = REG(nonterminals, terminals, rules, init)

    return reg




def main():
    dfa = test_dfa()
    dfa2 = test_dfa2()
    reg = test_reg()
    reg2 = test_reg2()
    parser = Parser()

    print("test: dfa_to_str")
    print(parser.dfa_to_str(dfa))
    print()
    print(parser.dfa_to_str(dfa, True))
    print()

    print("test: accepts")
    print(dfa.accepts("a"), dfa.accepts("aa"), dfa.accepts("ab"))

    print("test: add terminal, make total, is_total")
    b = Terminal("b")
    dfa.terminals.add(b)
    print(dfa.is_total())
    total_dfa = dfa.total()
    print(parser.dfa_to_str(total_dfa, True))
    print(total_dfa.is_total())
    print()

    print("test: complement")
    compl_dfa = dfa.complement()
    print(parser.dfa_to_str(compl_dfa, True))
    print()

    print("test: reg_to_str")
    print(parser.reg_to_str(reg))
    print()
    print(parser.reg_to_str(reg, True))
    print()

    print(parser.reg_to_str(reg2))
    print()
    print(parser.reg_to_str(reg2, True))
    print()

    print("test: second dfa_to_str")
    print(parser.dfa_to_str(dfa2, True))
    print()

    print("test: composition")
    union = dfa.composition(dfa, dfa2, op.Union)
    print(parser.dfa_to_str(union, True))
    print()
    intersection = dfa.composition(dfa, dfa2, op.Intersection)
    print(parser.dfa_to_str(intersection, True))
    print()

    print("test: add state, eliminate_unreachable")
    q_x = State("q_x")
    a = Terminal("a")
    dfa.states.add(q_x)
    dfa.final.add(q_x)
    dfa.transition[q_x,a] = q_x
    print(parser.dfa_to_str(dfa, True))
    print()
    eliminated = dfa.eliminate_unreachable()
    print(parser.dfa_to_str(eliminated, True))
    print()

    dfa_min = test_dfa_min()
    print(parser.dfa_to_str(dfa_min, True))

    print("test: minimize")
    minimal = dfa_min.minimize()
    print(parser.dfa_to_str(minimal, True))

    print("test: canonize")
    canonical = minimal.canonize()
    print(parser.dfa_to_str(canonical, True))

main()